@@ -4,12 +4,10 @@
 # https://opensource.org/licenses/MIT.
 
 import logging
-import os
 from datetime import datetime, timezone
 
 import pandas as pd
 from pystac import Catalog, Collection, Extent, Link, SpatialExtent, TemporalExtent
-from xcube.core.store import new_data_store
 
 from deep_code.constants import (
     DEEPESDL_COLLECTION_SELF_HREF,
@@ -17,6 +15,7 @@ from deep_code.constants import (
     PRODUCT_BASE_CATALOG_SELF_HREF,
     VARIABLE_BASE_CATALOG_SELF_HREF,
 )
+from deep_code.utils.helper import open_dataset
 from deep_code.utils.ogc_api_record import Theme, ThemeConcept
 from deep_code.utils.osc_extension import OscExtension
 
@@ -40,6 +39,9 @@ class OscDatasetStacGenerator:
         self,
         dataset_id: str,
         collection_id: str,
+        workflow_id: str,
+        workflow_title: str,
+        license_type: str,
         access_link: str | None = None,
         documentation_link: str | None = None,
         osc_status: str = "ongoing",
@@ -50,6 +52,9 @@ class OscDatasetStacGenerator:
     ):
         self.dataset_id = dataset_id
         self.collection_id = collection_id
+        self.workflow_id = workflow_id
+        self.workflow_title = workflow_title
+        self.license_type = license_type
         self.access_link = access_link or f"s3://deep-esdl-public/{dataset_id}"
         self.documentation_link = documentation_link
         self.osc_status = osc_status
@@ -58,69 +63,9 @@ class OscDatasetStacGenerator:
         self.osc_missions = osc_missions or []
         self.cf_params = cf_params or {}
         self.logger = logging.getLogger(__name__)
-        self.dataset = self._open_dataset()
+        self.dataset = open_dataset(dataset_id=dataset_id, logger=self.logger)
         self.variables_metadata = self.get_variables_metadata()
 
-    def _open_dataset(self):
-        """Open the dataset using a S3 store as a xarray Dataset."""
-
-        store_configs = [
-            {
-                "description": "Public store",
-                "params": {
-                    "storage_type": "s3",
-                    "root": "deep-esdl-public",
-                    "storage_options": {"anon": True},
-                },
-            },
-            {
-                "description": "Authenticated store",
-                "params": {
-                    "storage_type": "s3",
-                    "root": os.environ.get("S3_USER_STORAGE_BUCKET"),
-                    "storage_options": {
-                        "anon": False,
-                        "key": os.environ.get("S3_USER_STORAGE_KEY"),
-                        "secret": os.environ.get("S3_USER_STORAGE_SECRET"),
-                    },
-                },
-            },
-        ]
-
-        # Iterate through configurations and attempt to open the dataset
-        last_exception = None
-        tried_configurations = []
-        for config in store_configs:
-            tried_configurations.append(config["description"])
-            try:
-                self.logger.info(
-                    f"Attempting to open dataset with configuration: "
-                    f"{config['description']}"
-                )
-                store = new_data_store(
-                    config["params"]["storage_type"],
-                    root=config["params"]["root"],
-                    storage_options=config["params"]["storage_options"],
-                )
-                dataset = store.open_data(self.dataset_id)
-                self.logger.info(
-                    f"Successfully opened dataset with configuration: "
-                    f"{config['description']}"
-                )
-                return dataset
-            except Exception as e:
-                self.logger.error(
-                    f"Failed to open dataset with configuration: "
-                    f"{config['description']}. Error: {e}"
-                )
-                last_exception = e
-
-        raise ValueError(
-            f"Failed to open Zarr dataset with ID {self.dataset_id}. "
-            f"Tried configurations: {', '.join(tried_configurations)}. "
-            f"Last error: {last_exception}"
-        )
-
     def _get_spatial_extent(self) -> SpatialExtent:
         """Extract spatial extent from the dataset."""
         if {"lon", "lat"}.issubset(self.dataset.coords):
@@ -176,8 +121,7 @@ class OscDatasetStacGenerator:
     @staticmethod
     def _normalize_name(name: str | None) -> str | None:
         if name:
-            return (name.replace(" ", "-").
-                    replace("_", "-").lower())
+            return name.replace(" ", "-").replace("_", "-").lower()
         return None
 
     def _get_general_metadata(self) -> dict:
@@ -205,8 +149,9 @@ class OscDatasetStacGenerator:
         variable_ids = list(self.variables_metadata.keys())
         #  Remove 'crs' and 'spatial_ref' from the list if they exist, note that
         #  spatial_ref will be normalized to spatial-ref in variable_ids and skipped.
-        return [var_id for var_id in variable_ids if var_id not in ["crs",
-                                                                    "spatial-ref"]]
+        return [
+            var_id for var_id in variable_ids if var_id not in ["crs", "spatial-ref"]
+        ]
 
     def get_variables_metadata(self) -> dict[str, dict]:
         """Extract metadata for all variables in the dataset."""
@@ -232,7 +177,8 @@ class OscDatasetStacGenerator:
         if not gcmd_keyword_url:
             gcmd_keyword_url = input(
                 f"Enter GCMD keyword URL or a similar url for"
-                f" {var_metadata.get('variable_id')}: ").strip()
+                f" {var_metadata.get('variable_id')}: "
+            ).strip()
         var_catalog.add_link(
             Link(
                 rel="via",
@@ -326,23 +272,23 @@ class OscDatasetStacGenerator:
         return var_catalog
 
     def update_product_base_catalog(self, product_catalog_path) -> Catalog:
-            """Link product to base product catalog"""
-            product_base_catalog = Catalog.from_file(product_catalog_path)
-            product_base_catalog.add_link(
-                Link(
-                    rel="child",
-                    target=f"./{self.collection_id}/collection.json",
-                    media_type="application/json",
-                    title=self.collection_id,
-                )
+        """Link product to base product catalog"""
+        product_base_catalog = Catalog.from_file(product_catalog_path)
+        product_base_catalog.add_link(
+            Link(
+                rel="child",
+                target=f"./{self.collection_id}/collection.json",
+                media_type="application/json",
+                title=self.collection_id,
             )
-            # 'self' link: the direct URL where this JSON is hosted
-            product_base_catalog.set_self_href(PRODUCT_BASE_CATALOG_SELF_HREF)
-            return product_base_catalog
+        )
+        # 'self' link: the direct URL where this JSON is hosted
+        product_base_catalog.set_self_href(PRODUCT_BASE_CATALOG_SELF_HREF)
+        return product_base_catalog
 
-    def update_variable_base_catalog(self, variable_base_catalog_path, variable_ids) \
-            -> (
-            Catalog):
+    def update_variable_base_catalog(
+        self, variable_base_catalog_path, variable_ids
+    ) -> (Catalog):
         """Link product to base product catalog"""
         variable_base_catalog = Catalog.from_file(variable_base_catalog_path)
         for var_id in variable_ids:
@@ -387,7 +333,7 @@ class OscDatasetStacGenerator:
                     rel="related",
                     target=f"../../themes/{theme}/catalog.json",
                     media_type="application/json",
-                    title=f"Theme: {self.format_string(theme)}"
+                    title=f"Theme: {self.format_string(theme)}",
                 )
             )
         deepesdl_collection.set_self_href(DEEPESDL_COLLECTION_SELF_HREF)
@@ -431,7 +377,7 @@ class OscDatasetStacGenerator:
         concepts = [ThemeConcept(id=theme_str) for theme_str in osc_themes]
         return Theme(concepts=concepts, scheme=OSC_THEME_SCHEME)
 
-    def build_dataset_stac_collection(self) -> Collection:
+    def build_dataset_stac_collection(self, mode: str) -> Collection:
         """Build an OSC STAC Collection for the dataset.
 
         Returns:
@@ -534,10 +480,22 @@ class OscDatasetStacGenerator:
                 rel="related",
                 target="../../projects/deep-earth-system-data-lab/collection.json",
                 media_type="application/json",
-                title="Project: DeepESDL"
+                title="Project: DeepESDL",
             )
         )
 
+        if mode in "all":
+            collection.add_link(
+                Link(
+                    rel="related",
+                    target=f"../../experiments/{self.workflow_id}/record.json",
+                    media_type="application/json",
+                    title=f"Experiment: {self.workflow_title}",
+                )
+            )
+
+        collection.license = self.license_type
+
         # Validate OSC extension fields
         try:
             osc_extension.validate_extension()
